from typing import Generic, TypeVar

from rainbowadn.core import *
from rainbowadn.flow.core import *
from rainbowadn.flow.verification.core import *
from rainbowadn.nullability import *

__all__ = ('StateVerification',)

Header = TypeVar('Header')
State = TypeVar('State')
Link = TypeVar('Link')


class StateVerification(
    Verification[tuple[Nullable[Link], Link]],
    Generic[Header, State, Link]
):
    def __init__(
            self,
            mapper: Mapper[Link, tuple[Header, State]],
            verification: Verification[tuple[Nullable[State], Header, State]]
    ):
        assert isinstance(mapper, Mapper)
        assert isinstance(verification, Mapper)
        self.mapper = mapper
        self.verification = verification

    async def _state(self, chain: Nullable[Link]) -> Nullable[State]:
        assert isinstance(chain, Nullable)
        if chain.null():
            return Null()
        else:
            state: State
            _, state = await self.mapper.map(chain.resolve())
            return NotNull(state)

    async def _tuple(self, element: tuple[Nullable[Link], Link]) -> tuple[Nullable[State], Header, State]:
        assert isinstance(element, tuple)
        previous: Nullable[Link]
        chain: Link
        previous, chain = element
        assert isinstance(previous, Nullable)
        previous_state: Nullable[State]
        header: Header
        state: State
        previous_state, (header, state) = await gather(
            self._state(previous),
            self.mapper.map(chain),
        )
        assert isinstance(previous_state, Nullable)
        return previous_state, header, state

    async def verify(self, element: tuple[Nullable[Link], Link]) -> bool:
        assert isinstance(element, tuple)
        assert_true(await self.verification.verify(await self._tuple(element)))
        return True

    def loose(self) -> Verification[tuple[Nullable[Link], Link]]:
        return self
