from io import StringIO

from rainbowadn.core import *

__all__ = ('ClassReport',)


class ClassReport:
    def __init__(self):
        self.report: set[tuple[str, bytes, bytes]] = set()

    @classmethod
    def _tuple(cls, mentionable: Mentionable) -> tuple[str, bytes, bytes]:
        return type(mentionable).__name__, bytes(HashPoint.of(mentionable)), bytes(mentionable)

    def _cointains(self, mentionable: Mentionable) -> bool:
        return self._tuple(mentionable) in self.report

    async def _recurse(self, recursive: RecursiveMentionable):
        for mentioned in await gather(*map(HashPoint.resolve, recursive.points())):
            await self.walk(mentioned)

    def _contain(self, mentionable: Mentionable) -> None:
        self.report.add(self._tuple(mentionable))

    async def _save(self, mentionable: Mentionable):
        self._contain(mentionable)
        if isinstance(mentionable, RecursiveMentionable):
            await self._recurse(mentionable)

    async def walk(self, mentionable: Mentionable):
        if self._cointains(mentionable):
            return
        await self._save(mentionable)

    def format(self) -> str:
        s = StringIO()
        for type_, point, bytes_ in sorted(self.report):
            s.write(f'{type_:<32}:{point.hex()}:{bytes_.hex()}\n')
        return s.getvalue()
