import string
import unittest

from rainbowadn.atomic import *
from rainbowadn.collection.comparison import *
from rainbowadn.collection.trees.binary import *
from rainbowadn.core import *
from rainbowadn.encryption import *
from rainbowadn.instrument import *
from rainbowadn.testing.resolvers import *


class TestEncryption(unittest.IsolatedAsyncioTestCase):
    async def test_encryption(self):
        set_gather_linear()
        encrypt_ctr = Counter(Encrypted, 'encrypt')
        with self.subTest('setup'):
            key = b'a' * 32
        with self.subTest('create empty'):
            tree: ActiveBinaryTree[Plain, Integer] = ActiveBinaryTree.empty(
                AVL(PlainComparator(Replace())), Plain.factory()
            )
        with self.subTest('fill'):
            for char in string.ascii_uppercase:
                tree = await tree.add(HashPoint.of(Plain(char.encode())))
            print(await tree.reference.str(0))
        with self.subTest('encrypt'):
            target = tree.reference
            with encrypt_ctr:
                target = (await Encrypted.encrypt(target, key)).decrypted
                print(encrypt_ctr.counter)
            tree = tree.create(target)
            print(await tree.reference.str(0))
        with self.subTest('alter'):
            tree = await tree.add(HashPoint.of(Plain(b'NEWKEY')))
            tree = await tree.remove(HashPoint.of(Plain(b'F')))
            print(await tree.reference.str(0))
        with self.subTest('encrypt and migrate'):
            target = tree.reference
            with encrypt_ctr:
                eeed = await Encrypted.encrypt(target, key)
                print(encrypt_ctr.counter)
            print(await (await default_resolver().migrate_resolved(eeed)).decrypted.str(0))
        with self.subTest('re-encrypt'):
            new_key = b'b' * 32
            target = eeed.decrypted
            with encrypt_ctr:
                await Encrypted.encrypt(target, new_key)
                print(encrypt_ctr.counter)
